"""Copyright 2010 Adam Bark

This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""

from math import sin, cos, atan, pi, radians
import random
import pyglet
import kytten
import cocos
import physics
director = cocos.director.director
#director.init(width=800)
from pyglet.gl import *
from physics import Vector


class Path(cocos.layer.Layer):
    def __init__(self, simulation, short_step=1/60.):
	super(Path, self).__init__()
	self.step = short_step
	self.simulation = simulation
	self.vectors = []
	try:
	    position, velocity = self.simulation.step_force(self.step)
	except physics.Crash:
	    self.positions = (0,0,0)
	    self.path = pyglet.graphics.vertex_list(
	        1, ("v3f/stream", self.positions))
	    return
	self.positions = [position.x/2e4,position.y/2e4,-415]
	self.pre_vel = velocity
	self.path = pyglet.graphics.vertex_list(
	        1, ("v3f/stream", self.positions))

    def get_chunk(self, steps=5000, accuracy=10):
	for s in range(steps):
	    position, velocity = self.simulation.step(self.step)
	    if round(position.x/2e4, -1) != round(self.positions[-3], -1) or \
	       round(position.y/2e4, -1) != round(self.positions[-2], -1):
		self.vectors.append(position)
		self.positions.extend((position.x/2e4, position.y/2e4, -415))
	self.path = pyglet.graphics.vertex_list(len(self.positions)/3,
	                                        ("v3f/stream", self.positions))

    def drawit(self, x, y, z):
	try:
	    self.path.vertices[::3] = [
	        ((p*z)/10)-x for p in self.positions[::3]]
	    self.path.vertices[1::3] = [
	        ((p*z)/10)-y for p in self.positions[1::3]]
	    #self.path.draw(GL_POINTS)
	except ValueError:
	    #print len(self.path.vertices), len(self.positions)
	    pass
	self.path.draw(GL_LINE_STRIP)


class Craft(cocos.layer.Layer):
    def __init__(self, image):
	super(Craft, self).__init__()
	self.craft = cocos.sprite.Sprite(image)
	self.dd = 0
	self.px = 0
	self.py = 1e7
	self.vx = 5000
	self.vy = 0
	self.arrow = pyglet.graphics.vertex_list(
	    2, ("v2f/stream", (
	        self.px/2e5, self.py/2e5,
	        self.px/2e5+self.vx, self.py/2e5+self.vy)))

    def drawit(self, z):
	self.craft.rotation = self.dd
	self.craft.position = ((self.px*z)/4e5, (self.py*z)/4e5)
	self.craft.scale = z/26.65
	self.arrow.vertices = (
	    (self.px*z)/4e5, (self.py*z)/4e5,
	    ((self.px+self.vx*1000)*z)/4e5, ((self.py+self.vy*1000)*z)/4e5)
	self.craft.draw()
	self.arrow.draw(GL_LINE_STRIP)

class SimulationDemo(cocos.layer.Layer):
    def __init__(self, planet, simulation=None):
	print director.get_window_size()
	super(SimulationDemo, self).__init__()
	self.craft = Craft(pyglet.resource.image("craft.png"))
	self.simulation = simulation
	if simulation:
	    self.path = Path(self.simulation)
	else:
	    self.path = None
	self.planet = cocos.sprite.Sprite(planet)
	self.planet2 = planet
	self.z = 1
	self.x = 0
	self.y = 0
	director.window.push_handlers(self.on_mouse_scroll,
	                              self.on_mouse_drag,
	                              self.on_key_press)
	print self.camera.center
	print self.camera.eye
	print self.camera.up_vector

    def draw(self):
	glPushMatrix()
	self.camera.center = cocos.euclid.Point3(self.x, self.y, self.z)
	self.camera.eye = cocos.euclid.Point3(self.x, self.y, 415)
	self.camera.up_vector.z = self.z
	self.planet.scale = self.z/20.0
	self.camera.locate()
	self.planet.draw()
	self.craft.drawit(self.z)
	if self.path:
	    self.path.drawit(self.x, self.y, self.z)
	self.camera.restore()
	glPopMatrix()

    def on_mouse_scroll(self, x, y, scroll_x, scroll_y):
	if self.z + scroll_y > 0:
	    self.z += scroll_y

    def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
	#self.x = 320 - x
	#self.y = 240 - y
	self.x -= dx
	self.y -= dy

    def on_key_press(self, symbol, modifiers):
	if symbol == pyglet.window.key.C:
	    self.x = 0
	    self.y = 0


class PlanningRoom(cocos.layer.Layer):
    def __init__(self, sim_dem, theme_dir):
        super(PlanningRoom, self).__init__()
	self.player = pyglet.media.Player()
	self.explosion = pyglet.resource.media("BOOOOOMM-Katanama-7525.wav",
	                                       streaming=False)
	self.landing = [pyglet.resource.media("Machine_-public_d-283.wav",
	                                      streaming=False),
	                pyglet.resource.media("Mechanic-Public_D-139.wav",
	                                      streaming=False)]
	self.sim_dem = sim_dem
	self.sim_speed = 1/60.0
	self.steps = 500.0
	print theme_dir
	self.theme=kytten.Theme(theme_dir)
        self.batch = pyglet.graphics.Batch()
        self.fps = pyglet.clock.ClockDisplay()
	px = random.randint(-1e8, 1e8)
	py = random.randint(-1e8, 1e8)
	self.controls = {
	    "vx": kytten.Input(text="5000", length=4, on_input=self.text_inp),
	    "vy": kytten.Input(text="0", length=4, on_input=self.text_inp),
	    "px": kytten.Input(text=str(px), length=4, on_input=self.text_inp),
	    "py": kytten.Input(text=str(py), length=4, on_input=self.text_inp),
	    "dd": kytten.Input(text="0", length=4, on_input=self.change_dir),
	    "thrust": kytten.Slider(max_value=1e3, width=50),
	    "fuel": kytten.Slider(1.0, width=50, disabled=True),
	    "temp": kytten.Label(),
	    "speed": kytten.Label(),
	    "height": kytten.Label()
	}
        self.initial_prefs = kytten.Dialog(
	    kytten.TitleFrame(
	        "Flight Prefs",
	        kytten.VerticalLayout([
	            kytten.GridLayout([
	                [kytten.Widget(),
	                 kytten.Label("X"),
	                 kytten.Label("Y")],
	                [kytten.Label("Velocity"),
	                 self.controls["vx"],
	                 self.controls["vy"]],
	                [kytten.Label("Position"),
	                 self.controls["px"],
	                 self.controls["py"]]]),
	            kytten.HorizontalLayout(
	                [kytten.Label("Lift"),
	                 kytten.Slider(min_value=-2.0, max_value=10.0,
	                               on_set=self.apply_lift)])]
                    )),
	    window=director.window, batch=self.batch,
            anchor=kytten.ANCHOR_TOP_LEFT, theme=self.theme, movable=False)
	self.burn_settings = kytten.Dialog(
	    kytten.TitleFrame(
	        "Burn",
	        kytten.GridLayout([
	            [kytten.Label("Direction"), self.controls["dd"]],
	            [kytten.Label("Thrust"), self.controls["thrust"]]])),
	    window=director.window, batch=self.batch,
            anchor=kytten.ANCHOR_BOTTOM_LEFT, theme=self.theme, movable=False)
	#kytten.VerticalLayout([self.initial_prefs, self.burn_settings])
	self.hud = kytten.Dialog(
	    kytten.TitleFrame(
	        "HUD",
	        kytten.GridLayout([
	            [kytten.Label("Fuel"), self.controls["fuel"]],
	            [kytten.Label("Temp."), self.controls["temp"]],
	            [kytten.Label("Speed"), self.controls["speed"]],
	            [kytten.Label("Height"), self.controls["height"]]])),
	    window=director.window, batch=self.batch,
	    anchor=kytten.ANCHOR_TOP_RIGHT, theme=self.theme, movable=False)
	self.sim_controls = kytten.Dialog(
	    kytten.TitleFrame(
	        "Sim",
	        kytten.GridLayout([
	            [kytten.Label("Speed"),
	             kytten.Slider(min_value=1/100.0, max_value=2.0,
	                           on_set=self.change_sim_speed)],
	            [kytten.Label("Steps"),
	             kytten.Slider(min_value=1.0, max_value=20000,
	                           on_set=self.set_steps)]])),
	    window=director.window, batch=self.batch,
	    anchor=kytten.ANCHOR_BOTTOM_RIGHT, theme=self.theme, movable=False)
	self.buttons = kytten.Dialog(
	    kytten.HorizontalLayout([
	        kytten.Button("(Re)Start", on_click=self.run_sim),
	        kytten.Button("Continue", on_click=self.cont_sim),
	        kytten.Button("Stop", on_click=self.stop_sim)]),
	    window=director.window, batch=self.batch,
	    anchor=kytten.ANCHOR_BOTTOM, theme=self.theme, movable=False)
	self.text_inp(None)
	self.popup = None
	#self.tolerances = kytten.Dialog(
	#    kytten.TitleFrame(
	#        "Tolerances",
	#        kytten.GridLayout([
	#self.estimates = kytten.Dialog(
	#    kytten.TitleFrame(
	#        "Estimates",
	#        kytten.HorizontalLayout([
	#            kytten.Label
	#self.burn_settings = kytten.Dialog(
	#    kytten.Frame(
	#        kytten.VerticalLayout([
	#            kytten.Label("Thrust")


    def set_steps(self, steps):
	self.steps = steps

    def run_sim(self):
	#print self.controls["vx"].get_text()
	try:
	    craft = physics.Craft(
		1000, 1.0,
		Vector(float(self.controls["vx"].get_text()),
		       float(self.controls["vy"].get_text())),
		Vector(cos(-radians(float(self.controls["dd"].get_text()))),
		       sin(-radians(float(self.controls["dd"].get_text())))),
		Vector(float(self.controls["px"].get_text()),
		       float(self.controls["py"].get_text())),
		200)
	    #earth = physics.Planet(6e24, 6371e3, 101e3)
	    earth = physics.Planet(6e24, 4e6, 101e4)
	    self.sim_dem.path = Path(physics.Simulation(craft, earth))
	    self.sim_dem.path.simulation.craft.thrust = \
		    self.controls["thrust"].get_value()
	    pyglet.clock.schedule_interval(self.siminator,
	                                   self.sim_speed)
	except ValueError:
	    pass

    def change_sim_speed(self, value):
	self.sim_speed = value
	if self.sim_dem.path:
	    pyglet.clock.unschedule(self.siminator)
	    pyglet.clock.schedule_interval(self.siminator,
	                                   self.sim_speed)

    def stop_sim(self):
	if self.sim_dem.path:
	    phys_craft = self.sim_dem.path.simulation.craft
	    pyglet.clock.unschedule(self.siminator)
	    self.sim_dem.craft.px = phys_craft.position.x
	    self.sim_dem.craft.py = phys_craft.position.y
	    self.sim_dem.craft.vx = phys_craft.velocity.x
	    self.sim_dem.craft.vy = phys_craft.velocity.y

    def cont_sim(self):
	if self.sim_dem.path:
	    self.sim_dem.path.simulation.craft.thrust = \
	        self.controls["thrust"].get_value()
	    print self.sim_dem.path.simulation.craft.thrust
	    pyglet.clock.schedule_interval(self.siminator,
	                                   self.sim_speed)

    def siminator(self, dt):
	try:
	    self.sim_dem.path.get_chunk(self.steps)
	except (physics.Crash, physics.Explosion, physics.Landed) as e:
	    pyglet.clock.unschedule(self.siminator)
	    print "EXCEPTION!", e, type(e)
	    if type(e) == physics.Crash:
		self.explosion.play()
		self.popup = kytten.PopupMessage("You have crashed!",
		                                  window=director.window,
		                                  theme=self.theme,
		                                  on_escape=self.remove_popup)
	    elif type(e) == physics.Explosion:
		self.explosion.play()
		self.popup = kytten.PopupMessage(
		    "You pushed the limits! And lost",
		    window=director.window,
		    theme=self.theme,
		    on_escape=self.remove_popup)
	    else:
		player = pyglet.media.Player()
		for effect in self.landing:
		    player.queue(effect)
		player.play()
		self.popup = kytten.PopupMessage(
		    "Well done Captain. Succesful Landing",
		    window=director.window,
		    theme=self.theme,
		    on_escape=self.remove_popup)

    def remove_popup(self, var):
	self.popup = None

    def text_inp(self, text):
	try:
	    self.sim_dem.craft.px = float(self.controls["px"].get_text())
	    self.sim_dem.craft.py = float(self.controls["py"].get_text())
	    self.sim_dem.craft.vx = float(self.controls["vx"].get_text())
	    self.sim_dem.craft.vy = float(self.controls["vy"].get_text())
	    #self.sim_dem.craft.dd = float(self.controls["dd"].get_text())
	except ValueError:
	    pass

    def apply_lift(self, lift):
	if self.sim_dem.path:
	    self.sim_dem.path.simulation.craft.lift = lift

    def change_dir(self, text):
	try:
	    self.sim_dem.craft.dd = float(text)
	except ValueError:
	    pass

    def draw(self):
	if self.sim_dem.path:
	    craft = self.sim_dem.path.simulation.craft
	    self.controls["fuel"].set_pos(craft.fuel/4e6)
	    self.controls["temp"].set_text(unicode(round(craft.temperature)))
	    self.controls["speed"].set_text(
	        unicode(round(craft.velocity.magnitude())))
	    self.controls["height"].set_text(
	        unicode(round(craft.position.magnitude()/1000)))
	self.initial_prefs.on_update(None)
	self.buttons.on_update(None)
	self.burn_settings.on_update(None)
	self.hud.on_update(None)
	self.sim_controls.on_update(None)
	if self.popup:
	    self.popup.on_update(None)
	    self.popup.draw()
	#self.crash.on_update(None)
	#self.controls["fuel"].
	#print self.initial_prefs.content#_get_controls()[0][0].id
        self.batch.draw()
	#self.fps.draw()


if __name__ == "__main__":
    pyglet.resource.path = ['../data']
    pyglet.resource.reindex()
    cocos.director.director.init(width=800)
    #print director.window
    #director.window.register_event_type('on_update')
    #def update(dt):
#	director.window.dispatch_event('on_update', dt)
    #pyglet.clock.schedule(update)
    craft = physics.Craft(100000, 1.0, Vector(5000.0, 0.0),
                          Vector(1,0), Vector(-1e8, 6.5e7), 200)
    planet = physics.Planet(1e24, 1e7, 101e4)
    simul = physics.Simulation(craft, planet)
    sd = SimulationDemo(pyglet.resource.image("red_planet.png"))
    pr = PlanningRoom(sd)
    s = cocos.scene.Scene(sd, pr)
    cocos.director.director.run(s)